@@ -13,7 +13,7 @@ import numpy as np
 from datashader.colors import rgb
 from xarray import DataArray
 
-from numba import cuda
+import numba as nb
 import dask.array as da
 
 from numpy.random import RandomState
@@ -112,7 +112,7 @@ def _run_dask_numpy_bin(data, bins, new_values):
     return out
 
 
-@cuda.jit(device=True)
+@nb.cuda.jit(device=True)
 def _gpu_bin(data, bins, new_values):
     nbins = len(bins)
     val = data[0, 0]
@@ -139,9 +139,9 @@ def _gpu_bin(data, bins, new_values):
     return out
 
 
-@cuda.jit
+@nb.cuda.jit
 def _run_gpu_bin(data, bins, new_values, out):
-    i, j = cuda.grid(2)
+    i, j = nb.cuda.grid(2)
     if (i >= 0 and i < out.shape[0] and j >= 0 and j < out.shape[1]):
         out[i, j] = _gpu_bin(data[i:i+1, j:j+1], bins, new_values)
 
@@ -373,11 +373,11 @@ def quantile(agg: xr.DataArray,
     -----
         - Dask's percentile algorithm is approximate, while numpy's is exact.
         - This may cause some differences between results of vanilla numpy
-        and dask version of the input agg. (https://github.com/dask/dask/issues/3099) # noqa
+    and dask version of the input agg. (https://github.com/dask/dask/issues/3099) # noqa
 
     References
     ----------
-        -  PySAL: https://pysal.org/mapclassify/_modules/mapclassify/classifiers.html#Quantiles # noqa
+        - PySAL: https://pysal.org/mapclassify/_modules/mapclassify/classifiers.html#Quantiles # noqa
 
     Examples
     --------
@@ -440,30 +440,32 @@ def quantile(agg: xr.DataArray,
                      attrs=agg.attrs)
 
 
-@ngjit
+@nb.jit(nopython=True)
 def _run_numpy_jenks_matrices(data, n_classes):
     n_data = data.shape[0]
-    lower_class_limits = np.zeros((n_data + 1, n_classes + 1),
-                                  dtype=np.float64)
+    lower_class_limits = np.zeros(
+        (n_data + 1, n_classes + 1), dtype=np.float64
+    )
     lower_class_limits[1, 1:n_classes + 1] = 1.0
 
-    var_combinations = np.zeros((n_data + 1, n_classes + 1), dtype=np.float64)
+    var_combinations = np.zeros(
+        (n_data + 1, n_classes + 1), dtype=np.float64
+    )
     var_combinations[2:n_data + 1, 1:n_classes + 1] = np.inf
 
-    nl = data.shape[0] + 1
     variance = 0.0
 
-    for l in range(2, nl): # noqa
+    for l in range(2, n_data + 1): # noqa
         sum = 0.0
         sum_squares = 0.0
         w = 0.0
 
-        for m in range(1, l + 1):
+        for m in range(l):
             # `III` originally
-            lower_class_limit = l - m + 1
+            lower_class_limit = l - m
             i4 = lower_class_limit - 1
 
-            val = data[i4]
+            val = np.float64(data[i4])
 
             # here we're estimating variance for each potential classing
             # of the data, for each potential number of classes. `w`
@@ -479,14 +481,17 @@ def _run_numpy_jenks_matrices(data, n_classes):
             # of samples.
             variance = sum_squares - (sum * sum) / w
 
-            if i4 != 0:
-                for j in range(2, n_classes + 1):
-                    jm1 = j - 1
-                    if var_combinations[l, j] >= \
-                            (variance + var_combinations[i4, jm1]):
-                        lower_class_limits[l, j] = lower_class_limit
-                        var_combinations[l, j] = variance + \
-                            var_combinations[i4, jm1]
+            if i4 == 0:
+                continue
+            for j in range(2, n_classes + 1):
+                # if adding this element to an existing class
+                # will increase its variance beyond the limit, break
+                # the class at this point, setting the lower_class_limit
+                # at this point.
+                new_variance = variance + var_combinations[i4, j-1]
+                if var_combinations[l, j] >= new_variance:
+                    lower_class_limits[l, j] = lower_class_limit
+                    var_combinations[l, j] = new_variance
 
         lower_class_limits[l, 1] = 1.
         var_combinations[l, 1] = variance
@@ -494,7 +499,6 @@ def _run_numpy_jenks_matrices(data, n_classes):
     return lower_class_limits, var_combinations
 
 
-@ngjit
 def _run_numpy_jenks(data, n_classes):
     # ported from existing cython implementation:
     # https://github.com/perrygeo/jenks/blob/master/jenks.pyx
@@ -504,11 +508,10 @@ def _run_numpy_jenks(data, n_classes):
     lower_class_limits, _ = _run_numpy_jenks_matrices(data, n_classes)
 
     k = data.shape[0]
-    kclass = [0.] * (n_classes + 1)
-    count_num = n_classes
-
-    kclass[n_classes] = data[len(data) - 1]
+    kclass = np.zeros(n_classes + 1, dtype=np.float64)
     kclass[0] = data[0]
+    kclass[-1] = data[-1]
+    count_num = n_classes
 
     while count_num > 1:
         elt = int(lower_class_limits[k][count_num] - 2)
@@ -526,7 +529,9 @@ def _run_numpy_natural_break(data, num_sample, k):
         # randomly select sample from the whole dataset
         # create a pseudo random number generator
         generator = RandomState(1234567890)
-        idx = [i for i in range(0, data.size)]
+        idx = np.linspace(
+            0, data.size, data.size, endpoint=False, dtype=np.uint32
+        )
         generator.shuffle(idx)
         sample_idx = idx[:num_sample]
         sample_data = data.flatten()[sample_idx]
@@ -543,10 +548,12 @@ def _run_numpy_natural_break(data, num_sample, k):
                           'a long time.'.format(sample_data.size),
                           Warning)
 
-    # only include non-nan values
-    sample_data = np.asarray([i for i in sample_data if np.isfinite(i)])
+    if not isinstance(sample_data, np.ndarray):
+        sample_data = np.asarray(sample_data)
 
-    uv = np.unique(sample_data[np.isfinite(sample_data)])
+    # only include finite values
+    sample_data = sample_data[np.isfinite(sample_data)]
+    uv = np.unique(sample_data)
     uvk = len(uv)
 
     if uvk < k:
@@ -578,7 +585,7 @@ def _run_cupy_jenks_matrices(data, n_classes):
     nl = data.shape[0] + 1
     variance = 0.0
 
-    for l in range(2, nl): # noqa
+    for l in range(2, nl):  # noqa
         sum = 0.0
         sum_squares = 0.0
         w = 0.0
@@ -687,7 +694,7 @@ def _run_cupy_natural_break(data, num_sample, k):
 
 
 def natural_breaks(agg: xr.DataArray,
-                   num_sample: Optional[int] = None,
+                   num_sample: Optional[int] = 20000,
                    name: Optional[str] = 'natural_breaks',
                    k: int = 5) -> xr.DataArray:
     """
@@ -701,7 +708,7 @@ def natural_breaks(agg: xr.DataArray,
     agg : xarray.DataArray
         2D NumPy, CuPy, NumPy-backed Dask, or Cupy-backed Dask array
         of values to be reclassified.
-    num_sample : int, default=None
+    num_sample : int, default=20000
         Number of sample data points used to fit the model.
         Natural Breaks (Jenks) classification is indeed O(n²) complexity,
         where n is the total number of data points, i.e: `agg.size`
