@@ -133,21 +133,24 @@ def curvature(agg: xr.DataArray,
     .. plot::
        :include-source:
 
-        import datashader as ds
-        import numpy as np
         import matplotlib.pyplot as plt
+        import numpy as np
+        import xarray as xr
+
         from xrspatial import generate_terrain, curvature
 
-        # Create Canvas
-        W = 500
-        H = 300
-        cvs = ds.Canvas(plot_width = W,
-                        plot_height = H,
-                        x_range = (-20e6, 20e6),
-                        y_range = (-20e6, 20e6))
 
         # Generate Example Terrain
-        terrain_agg = generate_terrain(canvas = cvs)
+        W = 500
+        H = 300
+
+        template_terrain = xr.DataArray(np.zeros((H, W)))
+        x_range=(-20e6, 20e6)
+        y_range=(-20e6, 20e6)
+
+        terrain_agg = generate_terrain(
+            template_terrain, x_range=x_range, y_range=y_range
+        )
 
         # Edit Attributes
         terrain_agg = terrain_agg.assign_attrs(
@@ -171,15 +174,6 @@ def curvature(agg: xr.DataArray,
                 'units': 'rad',
             }
         )
-        # Where cells are extremely upwardly convex
-        curvature_agg.data = np.where(
-            np.logical_and(
-                curvature_agg.data > 3000,
-                curvature_agg.data < 4000,
-            ),
-            1,
-            np.nan,
-        )
 
         # Plot Terrain
         terrain_agg.plot(cmap = 'terrain', aspect = 2, size = 4)
@@ -197,14 +191,14 @@ def curvature(agg: xr.DataArray,
 
         >>> print(terrain_agg[200:203, 200:202])
         <xarray.DataArray 'Elevation' (lat: 3, lon: 2)>
-        array([[1264.02249454, 1261.94748873],
-               [1285.37061171, 1282.48046696],
-               [1306.02305679, 1303.40657515]])
+        array([[1264.02296597, 1261.947921  ],
+               [1285.37105519, 1282.48079719],
+               [1306.02339636, 1303.4069579 ]])
         Coordinates:
-          * lon      (lon) float64 -3.96e+06 -3.88e+06
-          * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
+        * lon      (lon) float64 -3.96e+06 -3.88e+06
+        * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
         Attributes:
-            res:            1
+            res:            (80000.0, 133333.3333333333)
             Description:    Example Terrain
             units:          km
             Max Elevation:  4000
@@ -213,14 +207,14 @@ def curvature(agg: xr.DataArray,
 
         >>> print(curvature_agg[200:203, 200:202])
         <xarray.DataArray 'Curvature' (lat: 3, lon: 2)>
-        array([[nan, nan],
-               [nan, nan],
-               [nan, nan]])
+        array([[-8.14280993e-08, -5.46873377e-08],
+               [ 2.85253552e-08,  1.24471207e-08],
+               [ 8.91172878e-08,  1.58492666e-07]])
         Coordinates:
-          * lon      (lon) float64 -3.96e+06 -3.88e+06
-          * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
+        * lon      (lon) float64 -3.96e+06 -3.88e+06
+        * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
         Attributes:
-            res:            1
+            res:            (80000.0, 133333.3333333333)
             Description:    Curvature
             units:          rad
             Max Elevation:  4000
