@@ -2,6 +2,7 @@ import xarray as xr
 import numpy as np
 
 from xrspatial.utils import ngjit
+from xrspatial.utils import get_dataarray_resolution
 
 from typing import Union, Optional
 
@@ -11,6 +12,27 @@ import warnings
 NONE = -1
 
 
+def _get_pixel_id(point, raster, xdim=None, ydim=None):
+    # get location in `raster` pixel space for `point` in y-x coordinate space
+    # point: (y, x) - coordinates of the point
+    # xdim: name of the x coordinate dimension in input `raster`.
+    # ydim: name of the x coordinate dimension in input `raster`
+
+    if ydim is None:
+        ydim = raster.dims[-2]
+    if xdim is None:
+        xdim = raster.dims[-1]
+    y_coords = raster.coords[ydim].data
+    x_coords = raster.coords[xdim].data
+
+    cellsize_x, cellsize_y = get_dataarray_resolution(raster, xdim, ydim)
+    py = int(abs(point[0] - y_coords[0]) / cellsize_y)
+    px = int(abs(point[1] - x_coords[0]) / cellsize_x)
+
+    # return index of row and column where the `point` located.
+    return py, px
+
+
 @ngjit
 def _is_not_crossable(cell_value, barriers):
     # nan cell is not walkable
@@ -23,6 +45,16 @@ def _is_not_crossable(cell_value, barriers):
     return False
 
 
+@ngjit
+def _is_inside(py, px, h, w):
+    inside = True
+    if px < 0 or px >= w:
+        inside = False
+    if py < 0 or py >= h:
+        inside = False
+    return inside
+
+
 @ngjit
 def _distance(x1, y1, x2, y2):
     # euclidean distance in pixel space from (y1, x1) to (y2, x2)
@@ -53,15 +85,6 @@ def _min_cost_pixel_id(cost, is_open):
     return py, px
 
 
-@ngjit
-def _find_pixel_id(x, y, xs, ys):
-    cellsize_y = ys[1] - ys[0]
-    cellsize_x = xs[1] - xs[0]
-    py = int((y - ys[0]) / cellsize_y)
-    px = int((x - xs[0]) / cellsize_x)
-    return py, px
-
-
 @ngjit
 def _find_nearest_pixel(py, px, data, barriers):
     # if the cell is already valid, return itself
@@ -210,20 +233,6 @@ def _a_star_search(data, path_img, start_py, start_px, goal_py, goal_px,
     return
 
 
-@ngjit
-def _is_inside(point, xmin, xmax, epsilon_x, ymin, ymax, epsilon_y):
-    # check if a point at (x, y) is within
-    # range from (xmin - epsilon_x, ymin - epsilon_y)
-    #       to   (xmax + epsilon_x, ymax + epsilon_y)
-
-    x, y = point
-    if (x < xmin - epsilon_x) or x > xmax + epsilon_x:
-        return False
-    if (y < ymin - epsilon_y) or y > ymax + epsilon_y:
-        return False
-    return True
-
-
 def a_star_search(surface: xr.DataArray,
                   start: Union[tuple, list, np.array],
                   goal: Union[tuple, list, np.array],
@@ -234,14 +243,13 @@ def a_star_search(surface: xr.DataArray,
                   snap_start: bool = False,
                   snap_goal: bool = False) -> xr.DataArray:
     """
-    Calculates, for all inner cells of an array, the 2D convolution of
     Calculate distance from a starting point to a goal through a
     surface graph. Starting location and goal location should be within
     the graph.
 
     A* is a modification of Dijkstra’s Algorithm that is optimized for
     a single destination. Dijkstra’s Algorithm can find paths to all
-    locations; A* finds paths to one location,or the closest of several
+    locations; A* finds paths to one location, or the closest of several
     locations. It prioritizes paths that seem to be leading closer to
     a goal.
 
@@ -254,9 +262,9 @@ def a_star_search(surface: xr.DataArray,
     surface : xr.DataArray
         2D array of values to bin.
     start : array-like object of 2 numeric elements
-        (x, y) or (lon, lat) coordinates of the starting point.
+        (y, x) or (lat, lon) coordinates of the starting point.
     goal : array like object of 2 numeric elements
-        (x, y) or (lon, lat) coordinates of the goal location.
+        (y, x) or (lat, lon) coordinates of the goal location.
     barriers : array like object, default=[]
         List of values inside the surface which are barriers
         (cannot cross).
@@ -280,135 +288,99 @@ def a_star_search(surface: xr.DataArray,
 
     References
     ----------
-        - Red Blob Games: https://www.redblobgames.com/pathfinding/a-star/implementation.html # noqa
-        - Nicholas Swift: https://medium.com/@nicholas.w.swift/easy-a-star-pathfinding-7e6689c7f7b2 # noqa
+        - Red Blob Games: https://www.redblobgames.com/pathfinding/a-star/implementation.html  # noqa
+        - Nicholas Swift: https://medium.com/@nicholas.w.swift/easy-a-star-pathfinding-7e6689c7f7b2  # noqa
 
     Examples
     --------
     .. plot::
        :include-source:
 
-        import datashader as ds
-        import matplotlib.pyplot as plt
-        from xrspatial import generate_terrain
-        from xrspatial.pathfinding import a_star_search
+        import numpy as np
+        import xarray as xr
+        from xrspatial import a_star_search
 
-        # Create Canvas
-        W = 500
-        H = 300
-        cvs = ds.Canvas(plot_width = W,
-                        plot_height = H,
-                        x_range = (-20e6, 20e6),
-                        y_range = (-20e6, 20e6))
+        agg = xr.DataArray(np.array([
+            [0, 1, 0, 0],
+             [1, 1, 0, 0],
+             [0, 1, 2, 2],
+             [1, 0, 2, 0],
+             [0, 2, 2, 2]
+        ]), dims=['lat', 'lon'])
 
-        # Generate Example Terrain
-        terrain_agg = generate_terrain(canvas = cvs)
+        height, width = agg.shape
+        _lon = np.linspace(0, width - 1, width)
+        _lat = np.linspace(height - 1, 0, height)
+        agg['lon'] = _lon
+        agg['lat'] = _lat
 
-        # Edit Attributes
-        terrain_agg = terrain_agg.assign_attrs(
-            {
-                'Description': 'Example Terrain',
-                'units': 'km',
-                'Max Elevation': '4000',
-            }
-        )
-
-        terrain_agg = terrain_agg.rename({'x': 'lon', 'y': 'lat'})
-        terrain_agg = terrain_agg.rename('Elevation')
-
-        # Choose Start and End Points
-        start = terrain_agg[3][100]
-        start_y = start.coords['lat'].data
-        start_x = start.coords['lon'].data
-
-        end = terrain_agg[298][250]
-        end_y = end.coords['lat'].data
-        end_x = end.coords['lon'].data
-
-        # Avoid Water
+        # set pixels with value 0 as barriers
         barriers = [0]
 
-        # Create Path Aggregate Array
-        path_agg = a_star_search(surface = terrain_agg,
-                                 start = (start_y, start_x),
-                                 goal = (end_y, end_x),
-                                 barriers = barriers,
-                                 x = 'lon',
-                                 y = 'lat')
+        start = (3, 0)
+        goal = (0, 1)
+        path_agg = a_star_search(agg, start, goal, barriers, 'lon', 'lat')
 
-        # Edit Attributes
-        path_agg = path_agg.rename('Distance')
+    ... sourcecode:: python
 
-        # Plot Terrain
-        terrain_agg.plot(cmap = 'terrain', aspect = 2, size = 4)
-        plt.title("Terrain")
-        plt.ylabel("latitude")
-        plt.xlabel("longitude")
-
-        # Plot Path
-        path_agg.plot(aspect = 2, size = 4)
-        plt.title("Path")
-        plt.ylabel("latitude")
-        plt.xlabel("longitude")
+        >>> print(path_agg)
+        <xarray.DataArray (lat: 5, lon: 4)>
+        array([[       nan,        nan,        nan,        nan],
+               [0.        ,        nan,        nan,        nan],
+               [       nan, 1.41421356,        nan,        nan],
+               [       nan,        nan, 2.82842712,        nan],
+               [       nan, 4.24264069,        nan,        nan]])
+        Coordinates:
+          * lon      (lon) float64 0.0 1.0 2.0 3.0
+          * lat      (lat) float64 4.0 3.0 2.0 1.0 0.0
     """
+
     if surface.ndim != 2:
-        raise ValueError("surface must be 2D")
+        raise ValueError("input `surface` must be 2D")
 
     if surface.dims != (y, x):
-        raise ValueError("surface.coords should be named as coordinates:"
+        raise ValueError("`surface.coords` should be named as coordinates:"
                          "({}, {})".format(y, x))
 
     if connectivity != 4 and connectivity != 8:
         raise ValueError("Use either 4 or 8-connectivity.")
 
-    y_coords = surface.coords[y].data
-    x_coords = surface.coords[x].data
-    epsilon_x = (x_coords[1] - x_coords[0]) / 2
-    epsilon_y = (y_coords[1] - y_coords[0]) / 2
+    # convert starting and ending point from geo coords to pixel coords
+    start_py, start_px = _get_pixel_id(start, surface, x, y)
+    goal_py, goal_px = _get_pixel_id(goal, surface, x, y)
 
+    h, w = surface.shape
     # validate start and goal locations are in the graph
-    if not _is_inside(start, x_coords[0], x_coords[-1], epsilon_x,
-                      y_coords[0], y_coords[-1], epsilon_y):
+    if not _is_inside(start_py, start_px, h, w):
         raise ValueError("start location outside the surface graph.")
 
-    if not _is_inside(goal, x_coords[0], x_coords[-1], epsilon_x,
-                      y_coords[0], y_coords[-1], epsilon_y):
+    if not _is_inside(goal_py, goal_px, h, w):
         raise ValueError("goal location outside the surface graph.")
 
     barriers = np.array(barriers)
 
-    # convert starting and ending point from geo coords to pixel coords
-    start_py, start_px = _find_pixel_id(start[0], start[1], x_coords, y_coords)
     if snap_start:
         # find nearest valid pixel to the start location
-        start_py, start_px = _find_nearest_pixel(start_py, start_px,
-                                                 surface.data, barriers)
+        start_py, start_px = _find_nearest_pixel(
+            start_py, start_px, surface.data, barriers
+        )
     if _is_not_crossable(surface.data[start_py, start_px], barriers):
-        with warnings.catch_warnings():
-            warnings.simplefilter("default")
-            warnings.warn('Start at a non crossable pixel', Warning)
+        warnings.warn("Start at a non crossable location", Warning)
 
-    goal_py, goal_px = _find_pixel_id(goal[0], goal[1], x_coords, y_coords)
     if snap_goal:
         # find nearest valid pixel to the goal location
-        goal_py, goal_px = _find_nearest_pixel(goal_py, goal_px,
-                                               surface.data, barriers)
+        goal_py, goal_px = _find_nearest_pixel(
+            goal_py, goal_px, surface.data, barriers
+        )
     if _is_not_crossable(surface.data[goal_py, goal_px], barriers):
-        with warnings.catch_warnings():
-            warnings.simplefilter("default")
-            warnings.warn('End at a non crossable pixel', Warning)
-
-    if start_py == NONE or goal_py == NONE:
-        with warnings.catch_warnings():
-            warnings.simplefilter("default")
-            warnings.warn('No valid pixels in input surface', Warning)
+        warnings.warn("End at a non crossable location", Warning)
 
     # 2d output image that stores the path
     path_img = np.zeros_like(surface, dtype=np.float64)
     # first, initialize all cells as np.nans
     path_img[:, :] = np.nan
 
-    if start_py != NONE or goal_py != NONE:
+    if start_py != NONE:
         neighbor_ys, neighbor_xs = _neighborhood_structure(connectivity)
         _a_star_search(surface.data, path_img, start_py, start_px,
                        goal_py, goal_px, barriers, neighbor_ys, neighbor_xs)
