@@ -224,16 +224,41 @@ def great_circle_distance(
 
 @ngjit
 def _distance(x1, x2, y1, y2, metric):
+
     if metric == EUCLIDEAN:
-        return euclidean_distance(x1, x2, y1, y2)
+        d = euclidean_distance(x1, x2, y1, y2)
 
-    if metric == GREAT_CIRCLE:
-        return great_circle_distance(x1, x2, y1, y2)
+    elif metric == GREAT_CIRCLE:
+        d = great_circle_distance(x1, x2, y1, y2)
 
-    if metric == MANHATTAN:
-        return manhattan_distance(x1, x2, y1, y2)
+    else:
+        # metric == MANHATTAN:
+        d = manhattan_distance(x1, x2, y1, y2)
 
-    return -1.0
+    return np.float32(d)
+
+
+@ngjit
+def _calc_direction(x1, x2, y1, y2):
+    # Calculate direction from (x1, y1) to a source cell (x2, y2).
+    # The output values are based on compass directions,
+    # 90 to the east, 180 to the south, 270 to the west, and 360 to the north,
+    # with 0 reserved for the source cell itself
+
+    if x1 == x2 and y1 == y2:
+        return 0
+
+    x = x2 - x1
+    y = y2 - y1
+    d = np.arctan2(-y, x) * 57.29578
+    if d < 0:
+        d = 90.0 - d
+    elif d > 90.0:
+        d = 360.0 - d + 90.0
+    else:
+        d = 90.0 - d
+
+    return np.float32(d)
 
 
 @ngjit
@@ -269,9 +294,9 @@ def _process_proximity_line(
     width : np.int64
         Image width.
         It is the number of pixels in the `source_line`.
-    max_distance : np.float64, maximum distance considered.
+    max_distance : np.float32, maximum distance considered.
     line_proximity : numpy.array
-        1d numpy array of type np.float64, calculated proximity from
+        1d numpy array of type np.float32, calculated proximity from
         source_line.
     values : numpy.array
         1d numpy array. A list of target pixel values
@@ -281,7 +306,7 @@ def _process_proximity_line(
     Returns
     -------
     self: numpy.array
-        1d numpy array of type np.float64. Corresponding proximity of
+        1d numpy array of type np.float32. Corresponding proximity of
         source_line.
     """
     start = width - 1
@@ -376,267 +401,6 @@ def _process_proximity_line(
     return
 
 
-@ngjit
-def _calc_direction(x1, x2, y1, y2):
-    # Calculate direction from (x1, y1) to a source cell (x2, y2).
-    # The output values are based on compass directions,
-    # 90 to the east, 180 to the south, 270 to the west, and 360 to the north,
-    # with 0 reserved for the source cell itself
-
-    if x1 == x2 and y1 == y2:
-        return 0
-
-    x = x2 - x1
-    y = y2 - y1
-    d = np.arctan2(-y, x) * 57.29578
-    if d < 0:
-        d = 90.0 - d
-    elif d > 90.0:
-        d = 360.0 - d + 90.0
-    else:
-        d = 90.0 - d
-    return d
-
-
-@ngjit
-def _process_numpy(
-    img,
-    x_coords,
-    y_coords,
-    target_values,
-    max_distance,
-    distance_metric,
-    process_mode
-):
-    # max_distance = _distance(x_coords[0], x_coords[-1],
-    #                          y_coords[0], y_coords[-1],
-    #                          distance_metric)
-
-    height, width = img.shape
-
-    pan_near_x = np.zeros(width, dtype=np.int64)
-    pan_near_y = np.zeros(width, dtype=np.int64)
-
-    # output of the function
-    img_distance = np.zeros(shape=(height, width), dtype=np.float64)
-    img_allocation = np.zeros(shape=(height, width), dtype=np.float64)
-    img_direction = np.zeros(shape=(height, width), dtype=np.float64)
-
-    # Loop from top to bottom of the image.
-    for i in prange(width):
-        pan_near_x[i] = -1
-        pan_near_y[i] = -1
-
-    # a single line of the input image img
-    scan_line = np.zeros(width, dtype=img.dtype)
-
-    # indexes of nearest pixels of current line scan_line
-    nearest_xs = np.zeros(width, dtype=np.int64)
-    nearest_ys = np.zeros(width, dtype=np.int64)
-
-    for line in prange(height):
-        # Read for target values.
-        for i in prange(width):
-            scan_line[i] = img[line][i]
-
-        line_proximity = np.zeros(width, dtype=np.float64)
-
-        for i in prange(width):
-            line_proximity[i] = -1.0
-            nearest_xs[i] = -1
-            nearest_ys[i] = -1
-
-        # left to right
-        _process_proximity_line(
-            scan_line,
-            x_coords,
-            y_coords,
-            pan_near_x,
-            pan_near_y,
-            True,
-            line,
-            width,
-            max_distance,
-            line_proximity,
-            nearest_xs,
-            nearest_ys,
-            target_values,
-            distance_metric,
-        )
-        for i in prange(width):
-            if nearest_xs[i] != -1 and line_proximity[i] >= 0:
-                img_allocation[line][i] = img[nearest_ys[i], nearest_xs[i]]
-                d = _calc_direction(
-                    x_coords[line, i],
-                    x_coords[nearest_ys[i], nearest_xs[i]],
-                    y_coords[line, i],
-                    y_coords[nearest_ys[i], nearest_xs[i]],
-                )
-                img_direction[line][i] = d
-
-        # right to left
-        for i in prange(width):
-            nearest_xs[i] = -1
-            nearest_ys[i] = -1
-
-        _process_proximity_line(
-            scan_line,
-            x_coords,
-            y_coords,
-            pan_near_x,
-            pan_near_y,
-            False,
-            line,
-            width,
-            max_distance,
-            line_proximity,
-            nearest_xs,
-            nearest_ys,
-            target_values,
-            distance_metric,
-        )
-
-        for i in prange(width):
-            img_distance[line][i] = line_proximity[i]
-            if nearest_xs[i] != -1 and line_proximity[i] >= 0:
-                img_allocation[line][i] = img[nearest_ys[i], nearest_xs[i]]
-                d = _calc_direction(
-                    x_coords[line, i],
-                    x_coords[nearest_ys[i], nearest_xs[i]],
-                    y_coords[line, i],
-                    y_coords[nearest_ys[i], nearest_xs[i]],
-                )
-                img_direction[line][i] = d
-
-    # Loop from bottom to top of the image.
-    for i in prange(width):
-        pan_near_x[i] = -1
-        pan_near_y[i] = -1
-
-    for line in prange(height - 1, -1, -1):
-        # Read first pass proximity.
-        for i in prange(width):
-            line_proximity[i] = img_distance[line][i]
-
-        # Read pixel target_values.
-        for i in prange(width):
-            scan_line[i] = img[line][i]
-
-        # Right to left
-        for i in prange(width):
-            nearest_xs[i] = -1
-            nearest_ys[i] = -1
-
-        _process_proximity_line(
-            scan_line,
-            x_coords,
-            y_coords,
-            pan_near_x,
-            pan_near_y,
-            False,
-            line,
-            width,
-            max_distance,
-            line_proximity,
-            nearest_xs,
-            nearest_ys,
-            target_values,
-            distance_metric,
-        )
-
-        for i in prange(width):
-            if nearest_xs[i] != -1 and line_proximity[i] >= 0:
-                img_allocation[line][i] = img[nearest_ys[i], nearest_xs[i]]
-                d = _calc_direction(
-                    x_coords[line, i],
-                    x_coords[nearest_ys[i], nearest_xs[i]],
-                    y_coords[line, i],
-                    y_coords[nearest_ys[i], nearest_xs[i]],
-                )
-
-                img_direction[line][i] = d
-
-        # Left to right
-        for i in prange(width):
-            nearest_xs[i] = -1
-            nearest_ys[i] = -1
-
-        _process_proximity_line(
-            scan_line,
-            x_coords,
-            y_coords,
-            pan_near_x,
-            pan_near_y,
-            True,
-            line,
-            width,
-            max_distance,
-            line_proximity,
-            nearest_xs,
-            nearest_ys,
-            target_values,
-            distance_metric,
-        )
-
-        # final post processing of distances
-        for i in prange(width):
-            if line_proximity[i] < 0:
-                line_proximity[i] = np.nan
-            else:
-                if nearest_xs[i] != -1 and line_proximity[i] >= 0:
-                    img_allocation[line][i] = img[nearest_ys[i], nearest_xs[i]]
-                    d = _calc_direction(
-                        x_coords[line, i],
-                        x_coords[nearest_ys[i], nearest_xs[i]],
-                        y_coords[line, i],
-                        y_coords[nearest_ys[i], nearest_xs[i]],
-                    )
-                    img_direction[line][i] = d
-
-        for i in prange(width):
-            img_distance[line][i] = line_proximity[i]
-
-    if process_mode == PROXIMITY:
-        return img_distance
-    elif process_mode == ALLOCATION:
-        return img_allocation
-    elif process_mode == DIRECTION:
-        return img_direction
-
-
-def _process_dask(
-    raster, xs, ys, target_values, max_distance, distance_metric, process_mode
-):
-    # calculate padding for width and height
-    if not np.isfinite(max_distance):
-        # consider all targets in the whole raster
-        height, width = raster.shape
-        pad_y = height - 1
-        pad_x = width - 1
-    else:
-        cellsize_x, cellsize_y = get_dataarray_resolution(raster)
-        pad_y = int(max_distance / cellsize_y + 0.5)
-        pad_x = int(max_distance / cellsize_x + 0.5)
-
-    out = da.map_overlap(
-        func=_process_numpy,
-        img=raster.data,
-        x_coords=xs,
-        y_coords=ys,
-        target_values=target_values,
-        max_distance=max_distance,
-        distance_metric=distance_metric,
-        process_mode=process_mode,
-        depth=(pad_y, pad_x),
-        boundary=np.nan,
-        meta=np.array(()),
-        trim=False,
-        chunks=raster.shape,  # noqa: workaround to ensure the output is a Dask array with correct shape
-    ).rechunk(raster.chunks)  # noqa: workaround to ensure the output chunksize is the same as of input raster
-
-    return out
-
-
 def _process(
     raster,
     x,
@@ -646,6 +410,7 @@ def _process(
     distance_metric,
     process_mode
 ):
+
     raster_dims = raster.dims
     if raster_dims != (y, x):
         raise ValueError(
@@ -667,30 +432,207 @@ def _process(
     if max_distance is None:
         max_distance = np.inf
 
+    max_possible_distance = _distance(
+        xs[0][0], xs[-1][-1], ys[0][0], ys[-1][-1], distance_metric
+    )
+
+    @ngjit
+    def _process_numpy(img, x_coords, y_coords):
+        height, width = img.shape
+        pan_near_x = np.zeros(width, dtype=np.int64)
+        pan_near_y = np.zeros(width, dtype=np.int64)
+
+        # output of the function
+        output_img = np.zeros(shape=(height, width), dtype=np.float32)
+        img_distance = np.zeros(shape=(height, width), dtype=np.float32)
+
+        # Loop from top to bottom of the image.
+        for i in prange(width):
+            pan_near_x[i] = -1
+            pan_near_y[i] = -1
+
+        # a single line of the input image img
+        scan_line = np.zeros(width, dtype=img.dtype)
+
+        # indexes of nearest pixels of current line scan_line
+        nearest_xs = np.zeros(width, dtype=np.int64)
+        nearest_ys = np.zeros(width, dtype=np.int64)
+
+        for line in prange(height):
+            # Read for target values.
+            for i in prange(width):
+                scan_line[i] = img[line][i]
+
+            line_proximity = np.zeros(width, dtype=np.float32)
+
+            for i in prange(width):
+                line_proximity[i] = -1.0
+                nearest_xs[i] = -1
+                nearest_ys[i] = -1
+
+            # left to right
+            _process_proximity_line(
+                scan_line, x_coords, y_coords,
+                pan_near_x, pan_near_y, True,
+                line, width, max_distance,
+                line_proximity, nearest_xs, nearest_ys,
+                target_values, distance_metric,
+            )
+            for i in prange(width):
+                if nearest_xs[i] != -1 and line_proximity[i] >= 0:
+                    if process_mode == ALLOCATION:
+                        output_img[line][i] = img[nearest_ys[i], nearest_xs[i]]
+                    elif process_mode == DIRECTION:
+                        output_img[line][i] = _calc_direction(
+                            x_coords[line, i],
+                            x_coords[nearest_ys[i], nearest_xs[i]],
+                            y_coords[line, i],
+                            y_coords[nearest_ys[i], nearest_xs[i]],
+                        )
+
+            # right to left
+            for i in prange(width):
+                nearest_xs[i] = -1
+                nearest_ys[i] = -1
+
+            _process_proximity_line(
+                scan_line, x_coords, y_coords,
+                pan_near_x, pan_near_y, False,
+                line, width, max_distance,
+                line_proximity, nearest_xs, nearest_ys,
+                target_values, distance_metric,
+            )
+
+            for i in prange(width):
+                img_distance[line][i] = line_proximity[i]
+
+                if nearest_xs[i] != -1 and line_proximity[i] >= 0:
+                    if process_mode == ALLOCATION:
+                        output_img[line][i] = img[nearest_ys[i], nearest_xs[i]]
+
+                    elif process_mode == DIRECTION:
+                        output_img[line][i] = _calc_direction(
+                            x_coords[line, i],
+                            x_coords[nearest_ys[i], nearest_xs[i]],
+                            y_coords[line, i],
+                            y_coords[nearest_ys[i], nearest_xs[i]],
+                        )
+
+        # Loop from bottom to top of the image.
+        for i in prange(width):
+            pan_near_x[i] = -1
+            pan_near_y[i] = -1
+
+        for line in prange(height - 1, -1, -1):
+            # Read first pass proximity.
+            for i in prange(width):
+                line_proximity[i] = img_distance[line][i]
+
+            # Read pixel target_values.
+            for i in prange(width):
+                scan_line[i] = img[line][i]
+
+            # Right to left
+            for i in prange(width):
+                nearest_xs[i] = -1
+                nearest_ys[i] = -1
+
+            _process_proximity_line(
+                scan_line, x_coords, y_coords,
+                pan_near_x, pan_near_y, False,
+                line, width, max_distance,
+                line_proximity, nearest_xs, nearest_ys,
+                target_values, distance_metric,
+            )
+
+            for i in prange(width):
+                if nearest_xs[i] != -1 and line_proximity[i] >= 0:
+                    if process_mode == ALLOCATION:
+                        output_img[line][i] = img[nearest_ys[i], nearest_xs[i]]
+
+                    elif process_mode == DIRECTION:
+                        output_img[line][i] = _calc_direction(
+                            x_coords[line, i],
+                            x_coords[nearest_ys[i], nearest_xs[i]],
+                            y_coords[line, i],
+                            y_coords[nearest_ys[i], nearest_xs[i]],
+                        )
+
+            # Left to right
+            for i in prange(width):
+                nearest_xs[i] = -1
+                nearest_ys[i] = -1
+
+            _process_proximity_line(
+                scan_line, x_coords, y_coords,
+                pan_near_x, pan_near_y, True,
+                line, width, max_distance,
+                line_proximity, nearest_xs, nearest_ys,
+                target_values, distance_metric,
+            )
+
+            # final post processing of distances
+            for i in prange(width):
+                if line_proximity[i] < 0:
+                    line_proximity[i] = np.nan
+                else:
+                    if nearest_xs[i] != -1 and line_proximity[i] >= 0:
+                        if process_mode == ALLOCATION:
+                            output_img[line][i] = img[
+                                nearest_ys[i], nearest_xs[i]]
+
+                        elif process_mode == DIRECTION:
+                            output_img[line][i] = _calc_direction(
+                                x_coords[line, i],
+                                x_coords[nearest_ys[i], nearest_xs[i]],
+                                y_coords[line, i],
+                                y_coords[nearest_ys[i], nearest_xs[i]],
+                            )
+
+            for i in prange(width):
+                img_distance[line][i] = line_proximity[i]
+
+        if process_mode == PROXIMITY:
+            return img_distance
+        else:
+            return output_img
+
+    def _process_dask(raster, xs, ys):
+
+        if max_distance >= max_possible_distance:
+            # consider all targets in the whole raster
+            # the data array is computed at once,
+            # make sure your data fit your memory
+            height, width = raster.shape
+            raster.data = raster.data.rechunk({0: height, 1: width})
+            xs = xs.rechunk({0: height, 1: width})
+            ys = ys.rechunk({0: height, 1: width})
+            pad_y = pad_x = 0
+        else:
+            cellsize_x, cellsize_y = get_dataarray_resolution(raster)
+            # calculate padding for each chunk
+            pad_y = int(max_distance / cellsize_y + 0.5)
+            pad_x = int(max_distance / cellsize_x + 0.5)
+
+        out = da.map_overlap(
+            _process_numpy,
+            raster.data, xs, ys,
+            depth=(pad_y, pad_x),
+            boundary=np.nan,
+            meta=np.array(()),
+        )
+        return out
+
     if isinstance(raster.data, np.ndarray):
         # numpy case
-        result = _process_numpy(
-            raster.data,
-            xs,
-            ys,
-            target_values,
-            max_distance,
-            distance_metric,
-            process_mode,
-        )
+        result = _process_numpy(raster.data, xs, ys)
+
     elif isinstance(raster.data, da.Array):
         # dask + numpy case
-        xs = da.from_array(xs, chunks=(raster.chunks[0]))
-        ys = da.from_array(ys, chunks=(raster.chunks[0]))
-        result = _process_dask(
-            raster,
-            xs,
-            ys,
-            target_values,
-            max_distance,
-            distance_metric,
-            process_mode
-        )
+        xs = da.from_array(xs, chunks=(raster.chunks))
+        ys = da.from_array(ys, chunks=(raster.chunks))
+        result = _process_dask(raster, xs, ys)
+
     return result
 
 
@@ -701,7 +643,7 @@ def proximity(
     x: str = "x",
     y: str = "y",
     target_values: list = [],
-    max_distance: float = None,
+    max_distance: float = np.inf,
     distance_metric: str = "EUCLIDEAN",
 ) -> xr.DataArray:
     """
@@ -732,20 +674,37 @@ def proximity(
     ----------
     raster : xr.DataArray
         2D array image with `raster.shape` = (height, width).
+
     x : str, default='x'
         Name of x-coordinates.
+
     y : str, default='y'
         Name of y-coordinates.
+
     target_values: list
         Target pixel values to measure the distance from. If this option
         is not provided, proximity will be computed from non-zero pixel
         values.
-    max_distance: float
+
+    max_distance: float, default=np.inf
         The maximum distance to search. Proximity distances greater than
-        this value will be set to a NaN value.
-        Note that this must be given in the same unit as input. For example,
-        if input raster is in latitude/longitude units, `max_distance` should
-        also be provided in latitude/longitude units.
+        this value will be set to NaN.
+        Should be given in the same distance unit as input.
+        For example, if input raster is in lat-lon and distances between points
+        within the raster is calculated using Euclidean distance metric,
+        `max_distance` should also be provided in lat-lon unit.
+        If using Great Circle distance metric, and thus all distances is in km,
+        `max_distance` should also be provided in kilometer unit.
+
+        When scaling with Dask, whether the function scales well depends on
+        the `max_distance` value. If `max_distance` is infinite by default,
+        this function only works on a single machine.
+        It should scale well, however, if `max_distance` is relatively small
+        compared to the maximum possible distance in two arbitrary points
+        in the input raster. Note that if `max_distance` is equal or larger
+        than the max possible distance between 2 arbitrary points in the input
+        raster, the input data array will be rechunked.
+
     distance_metric: str, default='EUCLIDEAN'
         The metric for calculating distance between 2 points.
         Valid distance metrics are:
@@ -777,9 +736,11 @@ def proximity(
             [0., 0., 0., 0., 0.],
             [0., 0., 0., 0., 0.]
         ])
+
+        n, m = data.shape
         raster = xr.DataArray(data, dims=['y', 'x'], name='raster')
         raster['y'] = np.arange(n)[::-1]
-        raster['x'] = np.arange(n)
+        raster['x'] = np.arange(m)
 
         proximity_agg = proximity(raster)
 
@@ -803,7 +764,7 @@ def proximity(
                [3.1622777, 2.236068 , 1.4142135, 1.       , 1.4142135],
                [3.6055512, 2.828427 , 2.236068 , 2.       , 2.236068 ],
                [4.2426405, 3.6055512, 3.1622777, 3.       , 3.1622777]],
-              dtype=float64)
+              dtype=float32)
         Coordinates:
           * y        (y) int64 4 3 2 1 0
           * x        (x) int64 0 1 2 3 4
@@ -834,7 +795,7 @@ def allocation(
     x: str = "x",
     y: str = "y",
     target_values: list = [],
-    max_distance: float = None,
+    max_distance: float = np.inf,
     distance_metric: str = "EUCLIDEAN",
 ):
     """
@@ -865,20 +826,37 @@ def allocation(
     ----------
     raster : xr.DataArray
         2D array of target data.
+
     x : str, default='x'
         Name of x-coordinates.
+
     y : str, default='y'
         Name of y-coordinates.
+
     target_values : list
         Target pixel values to measure the distance from. If this option
         is not provided, allocation will be computed from non-zero pixel
         values.
-    max_distance: float
+
+    max_distance: float, default=np.inf
         The maximum distance to search. Proximity distances greater than
-        this value will be set to a NaN value.
-        Note that this must be given in the same unit as input. For example,
-        if input raster is in latitude/longitude units, `max_distance` should
-        also be provided in latitude/longitude units.
+        this value will be set to NaN.
+        Should be given in the same distance unit as input.
+        For example, if input raster is in lat-lon and distances between points
+        within the raster is calculated using Euclidean distance metric,
+        `max_distance` should also be provided in lat-lon unit.
+        If using Great Circle distance metric, and thus all distances is in km,
+        `max_distance` should also be provided in kilometer unit.
+
+        When scaling with Dask, whether the function scales well depends on
+        the `max_distance` value. If `max_distance` is infinite by default,
+        this function only works on a single machine.
+        It should scale well, however, if `max_distance` is relatively small
+        compared to the maximum possible distance in two arbitrary points
+        in the input raster. Note that if `max_distance` is equal or larger
+        than the max possible distance between 2 arbitrary points in the input
+        raster, the input data array will be rechunked.
+
     distance_metric : str, default='EUCLIDEAN'
         The metric for calculating distance between 2 points. Valid
         distance metrics are: 'EUCLIDEAN', 'GREAT_CIRCLE', and 'MANHATTAN'.
@@ -909,9 +887,10 @@ def allocation(
             [0., 0., 0., 0., 0.],
             [0., 0., 0., 0., 0.]
         ])
+        n, m = data.shape
         raster = xr.DataArray(data, dims=['y', 'x'], name='raster')
         raster['y'] = np.arange(n)[::-1]
-        raster['x'] = np.arange(n)
+        raster['x'] = np.arange(m)
 
         allocation_agg = allocation(raster)
 
@@ -965,7 +944,7 @@ def direction(
     x: str = "x",
     y: str = "y",
     target_values: list = [],
-    max_distance: float = None,
+    max_distance: float = np.inf,
     distance_metric: str = "EUCLIDEAN",
 ):
     """
@@ -991,27 +970,45 @@ def direction(
     Similar to `proximity`, the implementation for NumPy-backed is ported
     from GDAL, which uses a dynamic programming approach to identify
     nearest target of a pixel from its surrounding neighborhood in a 3x3 window
-    The implementation for Dask-backed uses `sklearn.sklearn.neighbors.KDTree`
-    internally.
+    The implementation for Dask-backed uses `dask.map_overlap` to compute
+    proximity direction chunk by chunk by expanding the chunk's borders
+    to cover the `max_distance`.
 
     Parameters
     ----------
     raster : xr.DataArray
         2D array image with `raster.shape` = (height, width).
+
     x : str, default='x'
         Name of x-coordinates.
+
     y : str, default='y'
         Name of y-coordinates.
+
     target_values: list
         Target pixel values to measure the distance from. If this
         option is not provided, proximity will be computed from
         non-zero pixel values.
-    max_distance: float
+
+    max_distance: float, default=np.inf
         The maximum distance to search. Proximity distances greater than
-        this value will be set to a NaN value.
-        Note that this must be given in the same unit as input. For example,
-        if input raster is in latitude/longitude units, `max_distance` should
-        also be provided in latitude/longitude units.
+        this value will be set to NaN.
+        Should be given in the same distance unit as input.
+        For example, if input raster is in lat-lon and distances between points
+        within the raster is calculated using Euclidean distance metric,
+        `max_distance` should also be provided in lat-lon unit.
+        If using Great Circle distance metric, and thus all distances is in km,
+        `max_distance` should also be provided in kilometer unit.
+
+        When scaling with Dask, whether the function scales well depends on
+        the `max_distance` value. If `max_distance` is infinite by default,
+        this function only works on a single machine.
+        It should scale well, however, if `max_distance` is relatively small
+        compared to the maximum possible distance in two arbitrary points
+        in the input raster. Note that if `max_distance` is equal or larger
+        than the max possible distance between 2 arbitrary points in the input
+        raster, the input data array will be rechunked.
+
     distance_metric: str, default='EUCLIDEAN'
         The metric for calculating distance between 2 points.
         Valid distance_metrics are:
@@ -1043,9 +1040,10 @@ def direction(
             [0., 0., 0., 0., 0.],
             [1., 0., 0., 0., 0.]
         ])
+        n, m = data.shape
         raster = xr.DataArray(data, dims=['y', 'x'], name='raster')
         raster['y'] = np.arange(n)[::-1]
-        raster['x'] = np.arange(n)
+        raster['x'] = np.arange(m)
 
         direction_agg = direction(raster)
 
@@ -1069,7 +1067,7 @@ def direction(
                [ 90.      ,  90.      ,   0.      , 270.      , 270.      ],
                [360.      , 135.      , 180.      , 225.      , 243.43495 ],
                [  0.      , 270.      , 180.      , 206.56505 , 225.      ]],
-              dtype=float64)
+              dtype=float32)
         Coordinates:
           * y        (y) int64 4 3 2 1 0
           * x        (x) int64 0 1 2 3 4
