@@ -27,6 +27,14 @@ except ImportError:
 ngjit = jit(nopython=True, nogil=True)
 
 
+def has_cupy():
+    return cupy is not None
+
+
+def is_cupy_array(arr):
+    return has_cupy() and isinstance(arr, cupy.ndarray)
+
+
 def has_cuda():
     """Check for supported CUDA device. If none found, return False"""
     local_cuda = False
@@ -70,6 +78,15 @@ def cuda_args(shape):
     return bpg, tpb
 
 
+def calc_cuda_dims(shape):
+    threadsperblock = (32, 32)
+    blockspergrid = (
+        (shape[0] + (threadsperblock[0] - 1)) // threadsperblock[0],
+        (shape[1] + (threadsperblock[1] - 1)) // threadsperblock[1]
+    )
+    return blockspergrid, threadsperblock
+
+
 def is_cupy_backed(agg: xr.DataArray):
     try:
         return type(agg.data._meta).__module__.split(".")[0] == "cupy"
@@ -95,7 +112,11 @@ class ArrayTypeFunctionMapping(object):
             return self.numpy_func
 
         # cupy case
-        elif has_cuda() and isinstance(arr.data, cupy.ndarray):
+        elif (
+            has_cuda()
+            and cupy is not None
+            and isinstance(arr.data, cupy.ndarray)
+        ):
             return self.cupy_func
 
         # dask + cupy case
@@ -125,8 +146,52 @@ def validate_arrays(*arrays):
         if not isinstance(first_array.data, type(arrays[i].data)):
             raise ValueError("input arrays must have same type")
 
+    # ensure dask chunksizes of all arrays are the same
+    if isinstance(first_array.data, da.Array):
+        for i in range(1, len(arrays)):
+            if first_array.chunks != arrays[i].chunks:
+                arrays[i].data = arrays[i].data.rechunk(first_array.chunks)
 
-def calc_res(raster):
+
+def get_xy_range(raster, xdim=None, ydim=None):
+    """
+    Compute xrange and yrange for input `raster`
+
+    Parameters
+    ----------
+    raster: xarray.DataArray
+    xdim: str, default = None
+        Name of the x coordinate dimension in input `raster`.
+        If not provided, assume xdim is `raster.dims[-1]`
+    ydim: str, default = None
+        Name of the y coordinate dimension in input `raster`
+        If not provided, assume ydim is `raster.dims[-2]`
+
+    Returns
+    ----------
+    xrange, yrange
+        Tuple of tuples: (x, y-range).
+        xrange: tuple of (xmin, xmax)
+        yrange: tuple of (ymin, ymax)
+    """
+
+    if ydim is None:
+        ydim = raster.dims[-2]
+    if xdim is None:
+        xdim = raster.dims[-1]
+
+    xmin = raster[xdim].min().item()
+    xmax = raster[xdim].max().item()
+    ymin = raster[ydim].min().item()
+    ymax = raster[ydim].max().item()
+
+    xrange = (xmin, xmax)
+    yrange = (ymin, ymax)
+
+    return xrange, yrange
+
+
+def calc_res(raster, xdim=None, ydim=None):
     """
     Calculate the resolution of xarray.DataArray raster and return it
     as thetwo-tuple (xres, yres).
@@ -135,26 +200,31 @@ def calc_res(raster):
     ----------
     raster: xr.DataArray
         Input raster.
+    xdim: str, default = None
+        Name of the x coordinate dimension in input `raster`.
+        If not provided, assume xdim is `raster.dims[-1]`
+    ydim: str, default = None
+        Name of the y coordinate dimension in input `raster`
+        If not provided, assume ydim is `raster.dims[-2]`
 
     Returns
     -------
     xres, yres: tuple
         Tuple of (x-resolution, y-resolution).
-
-    Notes
-    -----
-        - Sourced from datashader.utils.
     """
+
     h, w = raster.shape[-2:]
-    ydim, xdim = raster.dims[-2:]
-    xcoords = raster[xdim].values
-    ycoords = raster[ydim].values
-    xres = (xcoords[-1] - xcoords[0]) / (w - 1)
-    yres = (ycoords[0] - ycoords[-1]) / (h - 1)
+    xrange, yrange = get_xy_range(raster, xdim, ydim)
+    xres = (xrange[-1] - xrange[0]) / (w - 1)
+    yres = (yrange[-1] - yrange[0]) / (h - 1)
     return xres, yres
 
 
-def get_dataarray_resolution(agg: xr.DataArray):
+def get_dataarray_resolution(
+    agg: xr.DataArray,
+    xdim: str = None,
+    ydim: str = None,
+):
     """
     Calculate resolution of xarray.DataArray.
 
@@ -162,12 +232,19 @@ def get_dataarray_resolution(agg: xr.DataArray):
     ----------
     agg: xarray.DataArray
         Input raster.
+    xdim: str, default = None
+        Name of the x coordinate dimension in input `raster`.
+        If not provided, assume xdim is `raster.dims[-1]`
+    ydim: str, default = None
+        Name of the y coordinate dimension in input `raster`
+        If not provided, assume ydim is `raster.dims[-2]`
 
     Returns
     -------
     cellsize_x, cellsize_y: tuple
         Tuple of (x cell size, y cell size).
     """
+
     # get cellsize out from 'res' attribute
     try:
         cellsize = agg.attrs.get("res")
@@ -182,10 +259,10 @@ def get_dataarray_resolution(agg: xr.DataArray):
             cellsize_x = cellsize
             cellsize_y = cellsize
         else:
-            cellsize_x, cellsize_y = calc_res(agg)
+            cellsize_x, cellsize_y = calc_res(agg, xdim, ydim)
 
     except Exception:
-        cellsize_x, cellsize_y = calc_res(agg)
+        cellsize_x, cellsize_y = calc_res(agg, xdim, ydim)
 
     return cellsize_x, cellsize_y
 
