@@ -3,7 +3,9 @@ from math import pi as PI
 from typing import Union
 
 import numpy as np
-from xrspatial.utils import ngjit
+from .utils import has_cupy, is_cupy_array, ngjit
+from .gpu_rtx import has_rtx
+
 
 import xarray
 
@@ -1333,8 +1335,8 @@ def _pop(stack):
 # function viewshed_in_memory()
 
 @ngjit
-def _viewshed(raster, vp_row, vp_col, vp_elev, vp_target, ew_res, ns_res,
-              event_rcts, event_aes, data, visibility_grid):
+def _viewshed_cpu_sweep(raster, vp_row, vp_col, vp_elev, vp_target, ew_res,
+                        ns_res, event_rcts, event_aes, data, visibility_grid):
     n_rows, n_cols = raster.shape
 
     # for e in event_list:
@@ -1501,163 +1503,25 @@ def _viewshed(raster, vp_row, vp_col, vp_elev, vp_target, ew_res, ns_res,
     return visibility_grid
 
 
-def viewshed(raster: xarray.DataArray,
-             x: Union[int, float],
-             y: Union[int, float],
-             observer_elev: float = OBS_ELEV,
-             target_elev: float = TARGET_ELEV) -> xarray.DataArray:
-    """
-    Calculate viewshed of a raster (the visible cells in the raster)
-    for the given viewpoint (observer) location.
+def _viewshed_cpu(
+    raster: xarray.DataArray,  # contains numpy array
+    x: Union[int, float],
+    y: Union[int, float],
+    observer_elev: float = OBS_ELEV,
+    target_elev: float = TARGET_ELEV,
+) -> xarray.DataArray:
 
-    Parameters
-    ----------
-    raster : xr.DataArray
-        Input raster image.
-    x : int, float
-        x-coordinate in data space of observer location.
-    y : int, float
-        y-coordinate in data space of observer location.
-    observer_elev : float
-        Observer elevation above the terrain.
-    target_elev : float
-        Target elevation offset above the terrain.
-
-    Returns
-    -------
-    viewshed: xarray.DataArray
-        A cell x in the visibility grid is recorded as follows:
-        If it is invisible, then x is set to INVISIBLE.
-        If it is visible,  then x is set to the vertical angle w.r.t
-        the viewpoint.
-
-    Examples
-    --------
-    .. plot::
-       :include-source:
-
-        import datashader as ds
-        import matplotlib.pyplot as plt
-        import pandas as pd
-        import numpy as np
-        from xrspatial import generate_terrain, viewshed
-
-        # Create Canvas
-        W = 500
-        H = 300
-        cvs = ds.Canvas(plot_width = W,
-                        plot_height = H,
-                        x_range = (-20e6, 20e6),
-                        y_range = (-20e6, 20e6))
-
-        # Generate Example Terrain
-        terrain_agg = generate_terrain(canvas = cvs)
-
-        # Edit Attributes
-        terrain_agg = terrain_agg.assign_attrs(
-            {
-                'Description': 'Example Terrain',
-                'units': 'km',
-                'Max Elevation': '4000',
-            }
-        )
-
-        terrain_agg = terrain_agg.rename({'x': 'lon', 'y': 'lat'})
-        terrain_agg = terrain_agg.rename('Elevation')
-
-        # Generate a Target Aggregate Array
-        volcano_agg = generate_terrain(canvas = cvs)
-        volcano_agg.data = np.where(np.logical_and(volcano_agg.data > 2800,
-                                                volcano_agg.data < 5000), 1, 0)
-        volcano_agg = volcano_agg.rename('volcano')
-        volcano_agg.values = volcano_agg.values.astype("float64")
-
-        # Edit Attributes
-        volcano_agg = volcano_agg.assign_attrs({'Description': 'Volcano'})
-        volcano_agg = volcano_agg.rename('Volcano')
-
-        # Create Observer Aggregate Array
-        OBSERVER_X = -12.5e6
-        OBSERVER_Y = 10e6
-        observer_df = pd.DataFrame({'x': [OBSERVER_X], 'y': [OBSERVER_Y]})
-        observer_agg = cvs.points(observer_df, 'x', 'y')
-
-        # Generate Viewshed Aggregate Array
-        view = viewshed(volcano_agg, x=OBSERVER_X, y=OBSERVER_Y)
-
-        # Plot Terrain
-        terrain_agg.plot(cmap = 'terrain', aspect = 2, size = 4)
-        plt.title("Terrain")
-        plt.ylabel("latitude")
-        plt.xlabel("longitude")
-
-        # Plot Volcano
-        volcano_agg.plot(cmap = 'Pastel1', aspect = 2, size = 4)
-        plt.title("Volcano")
-        plt.ylabel("latitude")
-        plt.xlabel("longitude")
-
-        # Plot Viewshed
-        view.plot(cmap = 'Greys', aspect = 2, size = 4)
-        plt.title("Viewshed")
-        plt.ylabel("latitude")
-        plt.xlabel("longitude")
-
-    .. sourcecode:: python
-
-        >>> print(terrain_agg[200:203, 200:202])
-        <xarray.DataArray 'Elevation' (lat: 3, lon: 2)>
-        array([[1264.02249454, 1261.94748873],
-               [1285.37061171, 1282.48046696],
-               [1306.02305679, 1303.40657515]])
-        Coordinates:
-          * lon      (lon) float64 -3.96e+06 -3.88e+06
-          * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
-        Attributes:
-            res:            1
-            Description:    Example Terrain
-            units:          km
-            Max Elevation:  4000
-
-        >>> print(volcano_agg[200:203, 200:202])
-        <xarray.DataArray 'Volcano' (y: 3, x: 2)>
-        array([[0., 0.],
-               [0., 0.],
-               [0., 0.]])
-        Coordinates:
-          * x        (x) float64 -3.96e+06 -3.88e+06
-          * y        (y) float64 6.733e+06 6.867e+06 7e+06
-        Attributes:
-            res:          1
-            Description:  Volcano
-
-        >>> print(view[200:203, 200:202])
-        <xarray.DataArray (y: 3, x: 2)>
-        array([[90., 90.],
-               [90., 90.],
-               [90., 90.]])
-        Coordinates:
-          * x        (x) float64 -3.96e+06 -3.88e+06
-          * y        (y) float64 6.733e+06 6.867e+06 7e+06
-        Attributes:
-            res:          1
-            Description:  Volcano
-    """
     height, width = raster.shape
 
     y_coords = raster.indexes.get('y').values
     x_coords = raster.indexes.get('x').values
 
     # validate x arg
-    if x < x_coords[0]:
-        raise ValueError("x argument outside of raster x_range")
-    elif x > x_coords[-1]:
+    if not (x_coords.min() <= x <= x_coords.max()):
         raise ValueError("x argument outside of raster x_range")
 
     # validate y arg
-    if y < y_coords[0]:
-        raise ValueError("y argument outside of raster y_range")
-    elif y > y_coords[-1]:
+    if not (y_coords.min() <= y <= y_coords.max()):
         raise ValueError("y argument outside of raster y_range")
 
     selection = raster.sel(x=[x], y=[y], method='nearest')
@@ -1711,12 +1575,175 @@ def viewshed(raster: xarray.DataArray,
     event_rcts = np.array(event_list[:, :3], dtype=np.int64)
     event_aes = np.array(event_list[:, 3:], dtype=np.float64)
 
-    viewshed_img = _viewshed(raster.values, viewpoint_row, viewpoint_col,
-                             viewpoint_elev, viewpoint_target, ew_res, ns_res,
-                             event_rcts, event_aes, data, visibility_grid)
+    viewshed_img = _viewshed_cpu_sweep(
+        raster.values, viewpoint_row, viewpoint_col, viewpoint_elev,
+        viewpoint_target, ew_res, ns_res, event_rcts, event_aes, data,
+        visibility_grid)
 
     visibility = xarray.DataArray(viewshed_img,
                                   coords=raster.coords,
                                   attrs=raster.attrs,
                                   dims=raster.dims)
     return visibility
+
+
+def viewshed(raster: xarray.DataArray,
+             x: Union[int, float],
+             y: Union[int, float],
+             observer_elev: float = OBS_ELEV,
+             target_elev: float = TARGET_ELEV) -> xarray.DataArray:
+    """
+    Calculate viewshed of a raster (the visible cells in the raster)
+    for the given viewpoint (observer) location.
+
+    Parameters
+    ----------
+    raster : xr.DataArray
+        Input raster image.
+    x : int, float
+        x-coordinate in data space of observer location.
+    y : int, float
+        y-coordinate in data space of observer location.
+    observer_elev : float
+        Observer elevation above the terrain.
+    target_elev : float
+        Target elevation offset above the terrain.
+
+    Returns
+    -------
+    viewshed: xarray.DataArray
+        A cell x in the visibility grid is recorded as follows:
+        If it is invisible, then x is set to INVISIBLE.
+        If it is visible,  then x is set to the vertical angle w.r.t
+        the viewpoint.
+
+    Examples
+    --------
+    .. plot::
+       :include-source:
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import pandas as pd
+        import xarray as xr
+
+        from xrspatial import generate_terrain, viewshed
+
+
+        # Generate Example Terrain
+        W = 500
+        H = 300
+
+        template_terrain = xr.DataArray(np.zeros((H, W)))
+        x_range=(-20e6, 20e6)
+        y_range=(-20e6, 20e6)
+
+        terrain_agg = generate_terrain(
+            template_terrain, x_range=x_range, y_range=y_range
+        )
+
+        # Edit Attributes
+        terrain_agg = terrain_agg.assign_attrs(
+            {
+                'Description': 'Example Terrain',
+                'units': 'km',
+                'Max Elevation': '4000',
+            }
+        )
+
+        terrain_agg = terrain_agg.rename({'x': 'lon', 'y': 'lat'})
+        terrain_agg = terrain_agg.rename('Elevation')
+
+        # Generate a Target Aggregate Array
+        volcano_agg = generate_terrain(
+            template_terrain, x_range=x_range, y_range=y_range
+        )
+        volcano_agg.data = np.where(np.logical_and(volcano_agg.data > 2800,
+                                                volcano_agg.data < 5000), 1, 0)
+        volcano_agg = volcano_agg.rename('volcano')
+        volcano_agg.values = volcano_agg.values.astype("float64")
+
+        # Edit Attributes
+        volcano_agg = volcano_agg.assign_attrs({'Description': 'Volcano'})
+        volcano_agg = volcano_agg.rename('Volcano')
+
+        # Generate Viewshed Aggregate Array
+        OBSERVER_X = -12.5e6
+        OBSERVER_Y = 10e6
+        view = viewshed(volcano_agg, x=OBSERVER_X, y=OBSERVER_Y)
+
+        # Plot Terrain
+        terrain_agg.plot(cmap = 'terrain', aspect = 2, size = 4)
+        plt.title("Terrain")
+        plt.ylabel("latitude")
+        plt.xlabel("longitude")
+
+        # Plot Volcano
+        volcano_agg.plot(cmap = 'Pastel1', aspect = 2, size = 4)
+        plt.title("Volcano")
+        plt.ylabel("latitude")
+        plt.xlabel("longitude")
+
+        # Plot Viewshed
+        view.plot(cmap = 'Greys', aspect = 2, size = 4)
+        plt.title("Viewshed")
+        plt.ylabel("latitude")
+        plt.xlabel("longitude")
+
+    .. sourcecode:: python
+
+        >>> print(terrain_agg[200:203, 200:202])
+        <xarray.DataArray 'Elevation' (lat: 3, lon: 2)>
+        array([[1264.02296597, 1261.947921  ],
+               [1285.37105519, 1282.48079719],
+               [1306.02339636, 1303.4069579 ]])
+        Coordinates:
+        * lon      (lon) float64 -3.96e+06 -3.88e+06
+        * lat      (lat) float64 6.733e+06 6.867e+06 7e+06
+        Attributes:
+            res:            (80000.0, 133333.3333333333)
+            Description:    Example Terrain
+            units:          km
+            Max Elevation:  4000
+
+        >>> print(volcano_agg[200:203, 200:202])
+        <xarray.DataArray 'Volcano' (y: 3, x: 2)>
+        array([[0., 0.],
+               [0., 0.],
+               [0., 0.]])
+        Coordinates:
+        * x        (x) float64 -3.96e+06 -3.88e+06
+        * y        (y) float64 6.733e+06 6.867e+06 7e+06
+        Attributes:
+            res:          (80000.0, 133333.3333333333)
+            Description:  Volcano
+
+        >>> print(view[200:203, 200:202])
+        <xarray.DataArray (y: 3, x: 2)>
+        array([[90., 90.],
+               [90., 90.],
+               [90., 90.]])
+        Coordinates:
+        * x        (x) float64 -3.96e+06 -3.88e+06
+        * y        (y) float64 6.733e+06 6.867e+06 7e+06
+        Attributes:
+            res:          (80000.0, 133333.3333333333)
+            Description:  Volcano
+    """
+
+    if isinstance(raster.data, np.ndarray):
+        return _viewshed_cpu(raster, x, y, observer_elev, target_elev)
+
+    elif has_cupy() and is_cupy_array(raster.data):
+        if has_rtx():
+            # Run on gpu
+            from .gpu_rtx.viewshed import viewshed_gpu
+            return viewshed_gpu(raster, x, y, observer_elev, target_elev)
+        else:
+            # Convert to numpy and run on cpu
+            import cupy as cp
+            raster.data = cp.asnumpy(raster.data)
+            return _viewshed_cpu(raster, x, y, observer_elev, target_elev)
+
+    else:
+        raise TypeError(f"Unsupported raster array type: {type(raster.data)}")
